    processor 6502
    include vcs.h

NUM_BLOCKS  = 12

;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

tmpVars     ds 4
frameCnt    .byte

pfLst       = .
pf1LLst      ds NUM_BLOCKS
pf2LLst      ds NUM_BLOCKS
pf2RLst      ds NUM_BLOCKS
pf1RLst      ds NUM_BLOCKS


;===============================================================================
; M A C R O S
;===============================================================================

  MAC BIT_B
    .byte   $24
  ENDM

  MAC BIT_W
    .byte   $2c
  ENDM

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================
    SEG     Bank0
    ORG     $f000

Start SUBROUTINE
    sei                         ;           Disable interrupts, if there are any.
    cld                         ;           Clear BCD math bit.
    ldx     #0
    txs
    pha                         ;           Set stack to beginning.
    txa
.clearLoop:
    pha
    dex
    bne     .clearLoop

    jsr     GameInit

.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop


GameInit SUBROUTINE
    lda     #$0e
    sta     COLUP0
    lda     #$94
    sta     COLUP1

    lda     #$40
    sta     COLUBK
    lda     #$48
    sta     COLUPF

    lda     #%001
    sta     CTRLPF

    ldx     #NUM_BLOCKS*4-1
.loopInit:
    lda     PFTbl,x
    sta     pfLst,x
    dex
    bpl     .loopInit

    rts


VerticalBlank SUBROUTINE
    lda     #2
    sta     WSYNC
    sta     VSYNC
    inc     frameCnt
    sta     WSYNC
    sta     WSYNC
    lsr
    ldx     #44
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts


GameCalc SUBROUTINE
    rts


DrawScreen SUBROUTINE
.tmpStack   = tmpVars
.tmpY       = tmpVars+1

    tsx
    stx     .tmpStack

    ldx     #227-1
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
    sta     VBLANK
    stx     TIM64T
;---------------------------------------------------------------

    ldx     #NUM_BLOCKS-2
    lda     #$40
    sta     PF0

; let background interleave:
    SLEEP   32
    lda     frameCnt
    lsr
    bcs     .kernel2
    SLEEP   3
    jmp     EnterKernel

.kernel2:
    SLEEP   2
    jmp     EnterKernel2

    align 256

; Alternative: Y als fixer Schleifenzhler und tmpPF statt tay:
; - total: +3*2 cylces
; - largest free block: 33 -> 38

.loopY2:                ;           @56
    SLEEP   4           ; 4
    bpl     .contY2     ; 3 =  7

.contLoop:              ;           @58
    lda     #3          ; 2
    sta     .tmpY       ; 3 =  5
.contY2:                ;           @63
    SLEEP   13          ;13 = 13    @76
;---------------------------------------
    SLEEP   20          ;20 = 20

    lda     pf1LLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2LLst,x   ; 4
    sta     PF2         ; 3 =  7

    lda     pf1RLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2RLst,x   ; 4
    sta     PF2         ; 3 =  7    @48

    SLEEP   12          ;12 = 12    @60

    lda     pf1LLst,x   ; 4
    sec                 ; 2
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf1LLst,x   ; 4
    sta     PF1         ; 3 =  3    @01
;---------------------------------------
    lda     pf2LLst,x   ; 4
    rol                 ; 2
    eor     #$ff        ; 2
    ora     pf2LLst,x   ; 4
    sta     PF2         ; 3 = 15    @16

    lda     pf2RLst,x   ; 4
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf2RLst,x   ; 4
    tay                 ; 2 = 14

    lda     pf1RLst,x   ; 4
    rol                 ; 2
    eor     #$ff        ; 2
    ora     pf1RLst,x   ; 4
    sta     PF1         ; 3 = 15    @45

    sty     PF2         ; 3 =  3    @48

    dec     .tmpY       ; 5
    bpl     .loopY2     ; 2= 7/8
    dex                 ; 2
    bmi     .exitX      ; 2

EnterKernel:
.loopX:                 ;           @59
    lda     #3          ; 2
    sta     .tmpY       ; 3
.contY1:                ;           @64
    SLEEP   12          ;12 = 12    @76
;---------------------------------------
    SLEEP   20          ;20 = 20

    lda     pf1LLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2LLst,x   ; 4
    sta     PF2         ; 3 =  7

    lda     pf1RLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2RLst,x   ; 4
    sta     PF2         ; 3 =  7    @48

    SLEEP   12          ;12 = 12    @60

    lda     pf1LLst+1,x ; 4
    sec                 ; 2
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf1LLst,x   ; 4
    sta     PF1         ; 3 =  3    @01
;---------------------------------------
    lda     pf2LLst+1,x ; 4
    rol                 ; 2
    eor     #$ff        ; 2
    ora     pf2LLst,x   ; 4
    sta     PF2         ; 3 = 15

    lda     pf2RLst+1,x ; 4
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf2RLst,x   ; 4
    tay                 ; 2 = 14

    lda     pf1RLst+1,x ; 4
    rol                 ; 2
    eor     #$ff        ; 2
    ora     pf1RLst,x   ; 4
    sta     PF1         ; 3 = 15

    sty     PF2         ; 3 =  3    @48

    dec     .tmpY       ; 5
    bpl     .loopY1     ; 2= 7/8

    jmp     .contLoop   ; 3

.loopY1:                ;           @56
    SLEEP   5           ; 5
    bpl     .contY1     ; 3 =  8

.exitX:
    sta     WSYNC
;---------------------------------------
    jmp     Exit

    align 256

Kernel2 SUBROUTINE
; create flickering background
.tmpStack   = tmpVars
.tmpY       = tmpVars+1

.loopY2:                ;           @56
    SLEEP   4           ; 4
    bpl     .contY2     ; 3 =  7

.contLoop:              ;           @58
    lda     #3          ; 2
    sta     .tmpY       ; 3 =  5
.contY2:                ;           @63
    SLEEP   4           ; 4 =  4    @67
;---------------------------------------
    lda     pf1LLst,x   ; 4
    sec                 ; 2
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf1LLst,x   ; 4
    sta     PF1         ; 3 = 17    @12

    lda     pf2LLst,x   ; 4
    rol                 ; 2
    eor     #$ff        ; 2
    ora     pf2LLst,x   ; 4
    sta     PF2         ; 3 = 15    @27

    lda     pf2RLst,x   ; 4
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf2RLst,x   ; 4
    tay                 ; 2 = 14

    lda     pf1RLst,x   ; 4
    rol                 ; 2
    eor     #$ff        ; 2 =  8

    sty     PF2         ; 3 =  3    @48

    ora     pf1RLst,x   ; 4
    sta     PF1         ; 3 =  7    @55

    SLEEP   21
;---------------------------------------
    SLEEP   20
    lda     pf1LLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2LLst,x   ; 4
    sta     PF2         ; 3 =  7

    lda     pf1RLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2RLst,x   ; 4
    sta     PF2         ; 3 =  7    @48

    dec     .tmpY       ; 5
    bpl     .loopY2     ; 2= 7/8
    dex                 ; 2
    bmi     .exitX      ; 2

EnterKernel2:
.loopX:                 ;           @59
    lda     #3          ; 2
    sta     .tmpY       ; 3
.contY1:                ;           @64
    SLEEP   3           ; 3 =  3    @67
;---------------------------------------
    lda     pf1LLst+1,x ; 4
    sec                 ; 2
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf1LLst,x   ; 4
    sta     PF1         ; 3 = 17

    lda     pf2LLst+1,x ; 4
    rol                 ; 2
    eor     #$ff        ; 2
    ora     pf2LLst,x   ; 4
    sta     PF2         ; 3 = 15

    lda     pf2RLst+1,x ; 4
    ror                 ; 2
    eor     #$ff        ; 2
    ora     pf2RLst,x   ; 4
    tay                 ; 2 = 14

    lda     pf1RLst+1,x ; 4
    rol                 ; 2
    eor     #$ff        ; 2 =  8

    sty     PF2         ; 3 =  3    @48

    ora     pf1RLst,x   ; 4
    sta     PF1         ; 3 =  7    @55

    SLEEP   21
;---------------------------------------
    SLEEP   20
    lda     pf1LLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2LLst,x   ; 4
    sta     PF2         ; 3 =  7

    lda     pf1RLst,x   ; 4
    sta     PF1         ; 3 =  7
    lda     pf2RLst,x   ; 4
    sta     PF2         ; 3 =  7    @48

    dec     .tmpY       ; 5
    bpl     .loopY1     ; 2= 7/8

    jmp     .contLoop   ; 3

.loopY1:                ;           @56
    SLEEP   5           ; 5
    bpl     .contY1     ; 3 =  8

.exitX:
    sta     WSYNC

Exit:
    inx
    stx     PF0
    stx     PF1
    stx     PF2

    ldx     #2
.waitScreen:
    lda     INTIM
    bpl     .waitScreen
    sta     WSYNC
    stx     VBLANK

    ldx     .tmpStack
    txs
    rts


OverScan SUBROUTINE
    lda     #36
    sta     TIM64T

.waitTim:
    lda     INTIM
    bne     .waitTim
    rts


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
    org     $f600

    align 256

PFTbl:
PF1LTbl:
    .byte   %00001111
    .byte   %00000000
    .byte   %00001100
    .byte   %00110011
    .byte   %00111100
    .byte   %00000000
    .byte   %00111111
    .byte   %11110011
    .byte   %11001100
    .byte   %00001111
    .byte   %00000000
    .byte   %11111111
PF2LTbl:
    .byte   %00000000
    .byte   %11000011
    .byte   %00000000
    .byte   %00110000
    .byte   %00111100
    .byte   %00000011
    .byte   %00110000
    .byte   %00001100
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   %11111111
PF2RTbl:
    .byte   %00000000
    .byte   %00000011
    .byte   %00111100
    .byte   %11000000
    .byte   %00110000
    .byte   %00110000
    .byte   %11001100
    .byte   %00000011
    .byte   %00110000
    .byte   %00111100
    .byte   %00000000
    .byte   %11111111
PF1RTbl:
    .byte   %11110011
    .byte   %00110000
    .byte   %11001100
    .byte   %11110011
    .byte   %11111100
    .byte   %11110000
    .byte   %00111111
    .byte   %11110011
    .byte   %11000000
    .byte   %00000000
    .byte   %00000000
    .byte   %11111111

    org $f7fc
    .word   Start
    .word   Start

